/* $Id: crires_model_qmetric.c,v 1.16 2012-01-12 17:57:59 bristowp Exp $
 *
 * This file is part of the CRIRES Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: bristowp $
 * $Date: 2012-01-12 17:57:59 $
 * $Revision: 1.16 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                                                   Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>

#include "crires_model_kernel.h"
#include "crires_model_io.h"
#include "crires_model_qmetric.h"

#include <stdio.h>
#include <math.h>
#include <float.h>
#include "crires_model_sa.h"

#include <stdlib.h>

#include "crires_model_cputime.h"


/* #include <rands.h> */
#include "crires_model_r250.h"

#include "irplib_utils.h"
#include "crires_load.h"
#include "crires_utils.h"
#include "crires_pfits.h"
#include "crires_dfs.h"

#ifdef _R250_H_
#define uniform(a,b)    ( a + (b - a) * dr250() )
#endif

/* #define MELT_ONLY  */
/* #define EQUIL_ONLY        */

#define BIG_VAL 3000000.0

#define QMAXIT 5000

/* for conversion from degree to radians */
/* static const DOUBLE DEG2RAD=M_PI/180.0; */
  
struct crires *qlocal_p_crires;
int qlocal_nparam;
double * qlocal_p_abest;
double * qlocal_p_amin;
double * qlocal_p_amax;
int * qlocal_p_aname;
ann_all_par *qlocal_p_all_par;

DOUBLE p_qobsx[250],p_qobsy[250],p_qobsf[250],p_q_penc[250],p_q_genc[250],p_qwl[250],p_q_slitw[250]; 
DOUBLE ** qznse;
int p_qobschip[250], p_qobsmode[250], qsp_array[250]; 
int qsize;

cpl_table * crires_model_qanneal_comp(ann_all_par *p_all_par,
				     int nparam,double * abest,
				     double * amin,
				     double * amax,
				     int * aname,
				     struct crires *p_crires,
				     int DS_size,
				     coord *msp_coord,
				     DOUBLE ** znse_ref)
{
  int ii;
  float z, t0, t;
  cpl_table * conf_tab ;
  double *a;
  a=(DOUBLE *)cpl_malloc(nparam*sizeof(DOUBLE));

  if ( SAInit(crires_model_qenergy,nparam) == 0 ) {
    cpl_msg_error(__func__, "trouble initializing in SAInit");
    return NULL;
  }

  qlocal_p_abest=abest;
  qlocal_p_amax=amax;
  qlocal_p_amin=amin;
  qlocal_p_aname=aname;
  qlocal_nparam=nparam;
  qlocal_p_all_par=p_all_par;
  qlocal_p_crires=p_crires;
  for (ii=0;ii<DS_size;ii++) {
    p_qwl[ii]=msp_coord[ii].wave;
    p_qobsx[ii]=msp_coord[ii].x;
    p_qobsy[ii]=msp_coord[ii].y;
    p_qobschip[ii]=msp_coord[ii].chip;
    p_qobsf[ii]=msp_coord[ii].flux;
    qsp_array[ii]=msp_coord[ii].slit_pos;
    p_qobsmode[ii]=msp_coord[ii].mode;
    p_q_penc[ii]=(double)(msp_coord[ii].penc);
    p_q_genc[ii]=(double)(msp_coord[ii].genc);
    //printf("penc genc %d %d \n",msp_coord[ii].penc,msp_coord[ii].genc);
    p_q_slitw[ii]=msp_coord[ii].slitw;
    //printf("slitw %lf\n",msp_coord[ii].slitw);
  }

  qsize = DS_size; 
  qznse=znse_ref;

  for (ii=0; ii<qlocal_nparam ; ii++) {
    a[ii]=qlocal_p_abest[ii];
  }

  z = crires_model_qenergy(a);
        
  SAinitial( a );
  SABoltzmann( 0.5 );

/*        SAjump( 150.0 ); */
        
  SAmelt( NO_VALUE );                /* melt the system */
  SAcurrent( a );

  z = crires_model_qenergy(a);

  t0 = SAtemperature( NO_VALUE );

  /* make it a bit warmer than melting temperature */
  t0 = 1.2 * SAtemperature( NO_VALUE );

  SAtemperature( t0 );

  t = SAanneal( QMAXIT );
  SAcurrent( a );

  z = crires_model_qenergy(a);
  /*set ord[59] to -999 so that io_output_cfg fn knows to not output
    all of the prism, grat and ord values*/
  qlocal_p_crires->ord[59]=-999;
  conf_tab = crires_model_io_output_cfg(qlocal_p_crires);
  crires_model_qoutput_data(); 
  
/*   for (ii = 0; ii < qsize; ii++) */
/*   { */
/*     abest[ii]=qlocal_p_abest[ii]; */
/*   } */
  SAfree();
  cpl_free(a);

  return conf_tab ;
}

void crires_model_qoutput_data(void)
{
  int counter;
  int ii,mm;

  double quad_diff; 
  double penc, genc,penc2, genc2;

/*   FILE* scan_out2; */
/*   scan_out2=fopen("coord_scan_new.dat","a+"); */

  if (crires_model_init(qlocal_p_crires)!=0) {
    cpl_msg_error(__func__, "Failed to initialise model" );
  }

  quad_diff=0;
  /*This can be done outside the ii loop since for the OTF case
    there is one unique penc and genc - NO LONGER TRUE!*/
  for (ii = 0; ii < qsize; ii++) {
    penc=p_q_penc[ii]-36000;
    penc2=penc*penc;
    qlocal_p_crires->mup1=qlocal_p_crires->penc_co1+
      qlocal_p_crires->penc_co2*penc+
      qlocal_p_crires->penc_co3*penc2;
      genc=p_q_genc[ii]-200000;
      genc2=genc*genc;
    qlocal_p_crires->mug=qlocal_p_crires->genc_co1+
      qlocal_p_crires->genc_co2*genc+
      qlocal_p_crires->genc_co3*genc2;
    qlocal_p_crires->es_x=qlocal_p_crires->slit[qsp_array[ii]];
    
    mm=p_qobsmode[ii];

    qlocal_p_crires->slitw=p_q_slitw[ii];
    if (crires_model_init(qlocal_p_crires)!=0) {
      cpl_msg_error(__func__, "Failed to initialise model" );
    }

    crires_model_eval(p_qwl[ii],mm,qznse,qlocal_p_crires);
    crires_model_detpix(qlocal_p_crires);
    if (p_qobschip[ii]==qlocal_p_crires->chippix[0]) {
      quad_diff+=
        ((p_qobsx[ii]-qlocal_p_crires->xpospix)*
         (p_qobsx[ii]-qlocal_p_crires->xpospix)+
         (p_qobsy[ii]-qlocal_p_crires->ypospix)*
         (p_qobsy[ii]-qlocal_p_crires->ypospix));
/*       printf("%d  %lf %lf  %lf %lf  %lf %lf  %d %d  %lf  %lf %lf %lf\n", */
/*              ii,p_qobsx[ii]-qlocal_p_crires->xpospix, */
/*              p_qobsy[ii]-qlocal_p_crires->ypospix,p_qobsx[ii], */
/*              qlocal_p_crires->xpospix,p_qobsy[ii],qlocal_p_crires->ypospix, */
/*              p_qobschip[ii],p_qobsmode[ii], p_qobsf[ii], p_qwl[ii]*1000.0, p_q_penc[ii], p_q_genc[ii]); */
    }
    counter++;
  }
/*   fclose(scan_out2); */
}

/* evaluate the sum squared error, the "energy" */
float crires_model_qenergy(double *a) {
  int flux_wt=0;
  static int counter;
  int mm;
  static int first_time;
  static float bestyet;
  double newval, size_use;
 
  int ii;
  float sum_nw = 0.0, val, val_max;
  float sum_fw = 0.0, val_fw, fw_max;
  float sum_x = 0.0, val_x, x_max= 0.0;
  float sum_y = 0.0, val_y, y_max= 0.0;
  float sum = 0.0;
  double penc, genc,penc2, genc2;


  for (ii=0; ii<qlocal_nparam; ii++) {
    if(first_time >0) {
      newval=qlocal_p_abest[ii]+((qlocal_p_amax[ii]-qlocal_p_amin[ii])/2.0)*a[ii];
    }
    else {
      newval=qlocal_p_abest[ii];
    }
    if (newval>qlocal_p_amax[ii] || newval<qlocal_p_amin[ii]) {
      sum_nw=HUGE_VAL;
      sum_fw=HUGE_VAL;
      sum_x=HUGE_VAL;
    }

    crires_model_qassign_crires(qlocal_p_aname[ii],newval);
  }
  if (crires_model_init(qlocal_p_crires)!=0) {
    cpl_msg_error(__func__, "Failed to initialise model" );
    return -1.0;
  }

  qlocal_p_crires->slit_on=0;
  qlocal_p_crires->slitw=1.0;
  fw_max=0.0;
  val_max=0.0;
  for (ii = 0; ii < qsize; ii++) {
    if (sum_nw<HUGE_VAL) {
      qlocal_p_crires->es_x=qlocal_p_crires->slit[qsp_array[ii]];
      mm=p_qobsmode[ii];

      penc=p_q_penc[ii]-36000;
      penc2=penc*penc;
      qlocal_p_crires->mup1=qlocal_p_crires->penc_co1+
	qlocal_p_crires->penc_co2*penc+
	qlocal_p_crires->penc_co3*penc2;
      genc=p_q_genc[ii]-200000;
      genc2=genc*genc;
      qlocal_p_crires->mug=qlocal_p_crires->genc_co1+
	qlocal_p_crires->genc_co2*genc+
	qlocal_p_crires->genc_co3*genc2;
      qlocal_p_crires->slitw=p_q_slitw[ii];
      if (crires_model_init(qlocal_p_crires)!=0) {
	cpl_msg_error(__func__, "Failed to initialise model" );
	return -1.0;
      }

      crires_model_eval(p_qwl[ii],mm,qznse,qlocal_p_crires);
      crires_model_detpix(qlocal_p_crires);
      if (p_qobschip[ii]==qlocal_p_crires->chippix[0]) {
        val_x=p_qobsx[ii]-qlocal_p_crires->xpospix;
	if (val_x<0.0) val_x=-val_x;
        val_y=p_qobsy[ii]-qlocal_p_crires->ypospix;
	if (val_y<0.0) val_y=-val_y;
	val=(val_x*val_x+val_y*val_y);
        val_fw=val*p_qobsf[ii];
      }
      else {
        val=BIG_VAL;
        val_x=BIG_VAL;
        val_y=BIG_VAL;
        val_fw=BIG_VAL;
      }
      if (val>val_max && val<BIG_VAL) {
	val_max=val;
	fw_max=val_fw;
	x_max=val_x;
	y_max=val_y;
      }
      sum_nw += val;
      sum_fw += val_fw;
      sum_x +=val_x;
      sum_y +=val_y;
    }
  }
  if (qsize>5) {
    sum_nw-=val_max;
    sum_fw-=fw_max;
    sum_x-=x_max;
    sum_y-=y_max;
    size_use=(float)(qsize-1);
  }
  else {
    size_use=(float)(qsize);
  }
  if (flux_wt==1) {
    sum=sum_fw;
  }
  else {
    sum=sum_x;
  }
  if (sqrt(sum/size_use)<bestyet) {
    cpl_msg_info(__func__,"Iterations: %d, Mean x residual: %lf", counter, sum_x/size_use);
/*     printf(" it:%d  sg:%f fk:%f  penc_co1:%lf genc_co1:%lf chipx[0]:%lf chipx[1]:%lf chipx[2]:%lf chipx[3]:%lf sum_nw:%f sum_nw_mean:%f mean x: %f mean y: %f \n",counter,qlocal_p_crires->sg, qlocal_p_crires->fk,qlocal_p_crires->penc_co1/DEG2RAD,qlocal_p_crires->genc_co1/DEG2RAD,qlocal_p_crires->chipx[0],qlocal_p_crires->chipx[1],qlocal_p_crires->chipx[2],qlocal_p_crires->chipx[3],sum_nw,sqrt(sum_nw/size_use), sum_x/size_use, sum_y/size_use); */
    bestyet=sqrt(sum/size_use);
  }
  counter++;
  if(first_time==0) {
    first_time++;
    bestyet=1000000;
  }
  return sum;
}

void crires_model_qassign_crires(int loc, double val)
{
  int compa;
  compa=strncmp((qlocal_p_all_par+loc)->name,"temper",6);
  if (compa==0) qlocal_p_crires->temper=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"es_xf",5);
  if (compa==0) qlocal_p_crires->es_xf=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"es_x_off",8);
  if (compa==0) qlocal_p_crires->es_x_off=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"es_y",4);
  if (compa==0) qlocal_p_crires->es_y=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"mues",4);
  if (compa==0) qlocal_p_crires->mues=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"nues",4);
  if (compa==0) qlocal_p_crires->nues=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"taues",5);
  if (compa==0) qlocal_p_crires->taues=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"es_s",4);
  if (compa==0) qlocal_p_crires->es_s=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"es_w",4);
  if (compa==0) qlocal_p_crires->es_w=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"fcp",3);
  if (compa==0) qlocal_p_crires->fcp=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"mup1",4);
  if (compa==0) qlocal_p_crires->mup1=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"axis_mup",8);
  if (compa==0) qlocal_p_crires->axis_mup=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"penc_co1",8);
  if (compa==0) qlocal_p_crires->penc_co1=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"penc_co2",8);
  if (compa==0) qlocal_p_crires->penc_co2=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"penc_co3",8);
  if (compa==0) qlocal_p_crires->penc_co3=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"nup1",4);
  if (compa==0) qlocal_p_crires->nup1=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"taup1",5);
  if (compa==0) qlocal_p_crires->taup1=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"axis_nup",8);
  if (compa==0) qlocal_p_crires->axis_nup=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"axis_taup",9);
  if (compa==0) qlocal_p_crires->axis_taup=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"mup2",4);
  if (compa==0) qlocal_p_crires->mup2=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"nup2",4);
  if (compa==0) qlocal_p_crires->nup2=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"taup2",5);
  if (compa==0) qlocal_p_crires->taup2=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"rind_delta",10);
  if (compa==0) qlocal_p_crires->rind_delta=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"muis",4);
  if (compa==0) qlocal_p_crires->muis=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"nuis",4);
  if (compa==0) qlocal_p_crires->nuis=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"tauis",5);
  if (compa==0) qlocal_p_crires->tauis=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"flipisx",7);
  if (compa==0) qlocal_p_crires->flipisx=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"flipisy",7);
  if (compa==0) qlocal_p_crires->flipisy=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"fis",3);
  if (compa==0) qlocal_p_crires->fis=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"slitx",5);
  if (compa==0) qlocal_p_crires->slitx=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"slity",5);
  if (compa==0) qlocal_p_crires->slity=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"slitrot",7);
  if (compa==0) qlocal_p_crires->slitrot=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"slitw",5);
  if (compa==0) qlocal_p_crires->slitw=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"slith",5);
  if (compa==0) qlocal_p_crires->slith=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"mug",3);
  if (compa==0) qlocal_p_crires->mug=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"axis_mug",8);
  if (compa==0) qlocal_p_crires->axis_mug=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"genc_co1",8);
  if (compa==0) qlocal_p_crires->genc_co1=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"genc_co2",8);
  if (compa==0) qlocal_p_crires->genc_co2=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"genc_co3",8);
  if (compa==0) qlocal_p_crires->genc_co3=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"nug",3);
  if (compa==0) qlocal_p_crires->nug=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"taug",4);
  if (compa==0) qlocal_p_crires->taug=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"axis_nug",8);
  if (compa==0) qlocal_p_crires->axis_nug=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"axis_taug",9);
  if (compa==0) qlocal_p_crires->axis_taug=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"sg",2);
  if (compa==0) qlocal_p_crires->sg=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"cam_dis",7);
  if (compa==0) qlocal_p_crires->cam_dis=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"muk",3);
  if (compa==0) qlocal_p_crires->muk=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"nuk",3);
  if (compa==0) qlocal_p_crires->nuk=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"tauk",4);
  if (compa==0) qlocal_p_crires->tauk=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"fk",2);
  if (compa==0) qlocal_p_crires->fk=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"mud",3);
  if (compa==0) qlocal_p_crires->mud=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"nud",3);
  if (compa==0) qlocal_p_crires->nud=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"taud",4);
  if (compa==0) qlocal_p_crires->taud=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pix",3);
  if (compa==0) qlocal_p_crires->pix=val;

  compa=strncmp((qlocal_p_all_par+loc)->name,"chipx[0]",8);
  if (compa==0) qlocal_p_crires->chipx[0]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"chipy[0]",8);
  if (compa==0) qlocal_p_crires->chipy[0]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"chiprot[0]",10);
  if (compa==0) qlocal_p_crires->chiprot[0]=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"chipx[1]",8);
  if (compa==0) qlocal_p_crires->chipx[1]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"chipy[1]",8);
  if (compa==0) qlocal_p_crires->chipy[1]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"chiprot[1]",10);
  if (compa==0) qlocal_p_crires->chiprot[1]=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"chipx[2]",8);
  if (compa==0) qlocal_p_crires->chipx[2]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"chipy[2]",8);
  if (compa==0) qlocal_p_crires->chipy[2]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"chiprot[2]",10);
  if (compa==0) qlocal_p_crires->chiprot[2]=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"chipx[3]",8);
  if (compa==0) qlocal_p_crires->chipx[3]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"chipy[3]",8);
  if (compa==0) qlocal_p_crires->chipy[3]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"chiprot[3]",10);
  if (compa==0) qlocal_p_crires->chiprot[3]=val*DEG2RAD;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_xx[0]",10);
  if (compa==0) qlocal_p_crires->pc_x_xx[0]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_x1[0]",10);
  if (compa==0) qlocal_p_crires->pc_x_x1[0]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_yy[0]",10);
  if (compa==0) qlocal_p_crires->pc_x_yy[0]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_y1[0]",10);
  if (compa==0) qlocal_p_crires->pc_x_y1[0]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_xy[0]",10);
  if (compa==0) qlocal_p_crires->pc_x_xy[0]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_xx[0]",10);
  if (compa==0) qlocal_p_crires->pc_y_xx[0]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_x1[0]",10);
  if (compa==0) qlocal_p_crires->pc_y_x1[0]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_yy[0]",10);
  if (compa==0) qlocal_p_crires->pc_y_yy[0]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_y1[0]",10);
  if (compa==0) qlocal_p_crires->pc_y_y1[0]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_xy[0]",10);
  if (compa==0) qlocal_p_crires->pc_y_xy[0]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_xx[1]",10);
  if (compa==0) qlocal_p_crires->pc_x_xx[1]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_x1[1]",10);
  if (compa==0) qlocal_p_crires->pc_x_x1[1]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_yy[1]",10);
  if (compa==0) qlocal_p_crires->pc_x_yy[1]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_y1[1]",10);
  if (compa==0) qlocal_p_crires->pc_x_y1[1]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_xy[1]",10);
  if (compa==0) qlocal_p_crires->pc_x_xy[1]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_xx[1]",10);
  if (compa==0) qlocal_p_crires->pc_y_xx[1]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_x1[1]",10);
  if (compa==0) qlocal_p_crires->pc_y_x1[1]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_yy[1]",10);
  if (compa==0) qlocal_p_crires->pc_y_yy[1]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_y1[1]",10);
  if (compa==0) qlocal_p_crires->pc_y_y1[1]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_xy[1]",10);
  if (compa==0) qlocal_p_crires->pc_y_xy[1]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_xx[2]",10);
  if (compa==0) qlocal_p_crires->pc_x_xx[2]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_x1[2]",10);
  if (compa==0) qlocal_p_crires->pc_x_x1[2]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_yy[2]",10);
  if (compa==0) qlocal_p_crires->pc_x_yy[2]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_y1[2]",10);
  if (compa==0) qlocal_p_crires->pc_x_y1[2]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_xy[2]",10);
  if (compa==0) qlocal_p_crires->pc_x_xy[2]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_xx[2]",10);
  if (compa==0) qlocal_p_crires->pc_y_xx[2]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_x1[2]",10);
  if (compa==0) qlocal_p_crires->pc_y_x1[2]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_yy[2]",10);
  if (compa==0) qlocal_p_crires->pc_y_yy[2]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_y1[2]",10);
  if (compa==0) qlocal_p_crires->pc_y_y1[2]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_xy[2]",10);
  if (compa==0) qlocal_p_crires->pc_y_xy[2]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_xx[3]",10);
  if (compa==0) qlocal_p_crires->pc_x_xx[3]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_x1[3]",10);
  if (compa==0) qlocal_p_crires->pc_x_x1[3]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_yy[3]",10);
  if (compa==0) qlocal_p_crires->pc_x_yy[3]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_y1[3]",10);
  if (compa==0) qlocal_p_crires->pc_x_y1[3]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_x_xy[3]",10);
  if (compa==0) qlocal_p_crires->pc_x_xy[3]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_xx[3]",10);
  if (compa==0) qlocal_p_crires->pc_y_xx[3]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_x1[3]",10);
  if (compa==0) qlocal_p_crires->pc_y_x1[3]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_yy[3]",10);
  if (compa==0) qlocal_p_crires->pc_y_yy[3]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_y1[3]",10);
  if (compa==0) qlocal_p_crires->pc_y_y1[3]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"pc_y_xy[3]",10);
  if (compa==0) qlocal_p_crires->pc_y_xy[3]=val;

  compa=strncmp((qlocal_p_all_par+loc)->name,"det_array_x",11);
  if (compa==0) qlocal_p_crires->det_array_x=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"det_array_y",11);
  if (compa==0) qlocal_p_crires->det_array_y=val;

  compa=strncmp((qlocal_p_all_par+loc)->name,"offx",4);
  if (compa==0) qlocal_p_crires->offx=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"offy",4);
  if (compa==0) qlocal_p_crires->offy=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"flipx",5);
  if (compa==0) qlocal_p_crires->flipx=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"flipy",5);
  if (compa==0) qlocal_p_crires->flipy=val;

  compa=strncmp((qlocal_p_all_par+loc)->name,"simpl",5);
  if (compa==0) qlocal_p_crires->simpl=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"nx",2);
  if (compa==0) qlocal_p_crires->nx=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"ny",2);
  if (compa==0) qlocal_p_crires->ny=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"uvab",4);
  if (compa==0) qlocal_p_crires->uvab=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"slitp",5);
  if (compa==0) qlocal_p_crires->slitp=val;

  compa=strncmp((qlocal_p_all_par+loc)->name,"slit[1]",7);
  if (compa==0) qlocal_p_crires->slit[1]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"slit[2]",7);
  if (compa==0) qlocal_p_crires->slit[2]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"slit[3]",7);
  if (compa==0) qlocal_p_crires->slit[3]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"slit[4]",7);
  if (compa==0) qlocal_p_crires->slit[4]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"slit[5]",7);
  if (compa==0) qlocal_p_crires->slit[5]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"slit[6]",7);
  if (compa==0) qlocal_p_crires->slit[6]=val;
  compa=strncmp((qlocal_p_all_par+loc)->name,"slit[7]",7);
  if (compa==0) qlocal_p_crires->slit[7]=val;
}
